# The LearningOnline Network with CAPA
#
# $Id: lonstatistics.pm,v 1.111 2004/09/24 19:41:17 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# (Navigate problems for statistical reports
#
###

=pod

=head1 NAME

lonstatistics

=head1 SYNOPSIS

Main handler for statistics and chart.

=over 4

=cut

package Apache::lonstatistics;

use strict;
use Apache::Constants qw(:common :http);
use vars qw(
    @FullClasslist 
    @Students
    @Sections 
    @SelectedSections
    %StudentData
    @StudentDataOrder
    @SelectedStudentData
    $top_map 
    @Sequences 
    @SelectedMaps
    @Assessments
    $enrollment_status);

use Apache::lonnet();
use Apache::lonhomework;
use Apache::loncommon;
use Apache::loncoursedata;
use Apache::lonhtmlcommon;
use Apache::lonmysql;
use Apache::lonlocal;
use Time::HiRes;
#
# Statistics Packages
use Apache::lonproblemanalysis();
use Apache::lonsubmissiontimeanalysis();
use Apache::loncorrectproblemplot();
use Apache::lonproblemstatistics();
use Apache::lonstudentassessment();
use Apache::lonpercentage;
use Apache::lonstudentsubmissions();
use Apache::lonsurveyreports();

#######################################################
#######################################################

=pod

=item Package Variables

=item @FullClasslist The full classlist

=item @Students The students we are concerned with for this invocation

=item @Sections The sections available in this class

=item $curr_student The student currently being examined

=item $prev_student The student previous in the classlist

=item $next_student The student next in the classlist

=over

=cut 

#######################################################
#######################################################
#
# Classlist variables
#
my $curr_student;
my $prev_student;
my $next_student;

#######################################################
#######################################################

=pod

=item &clear_classlist_variables()

undef the following package variables:

=over

=item @FullClasslist

=item @Students

=item @Sections

=item @SelectedSections

=item %StudentData

=item @StudentDataOrder

=item @SelectedStudentData

=item $curr_student

=item $prev_student

=item $next_student

=back

=cut

#######################################################
#######################################################
sub clear_classlist_variables {
    undef(@FullClasslist);
    undef(@Students);
    undef(@Sections);
    undef(@SelectedSections);
    undef(%StudentData);
    undef(@SelectedStudentData);
    undef($curr_student);
    undef($prev_student);
    undef($next_student);
}

#######################################################
#######################################################

=pod

=item &PrepareClasslist()

Build up the classlist information.  The classlist information is kept in
the following package variables:

=over

=item @FullClasslist

=item @Students

=item @Sections

=item @SelectedSections

=item %StudentData

=item @SelectedStudentData

=item $curr_student

=item $prev_student

=item $next_student

=back

$curr_student, $prev_student, and $next_student may not be defined, depending
upon the calling context.

=cut

#######################################################
#######################################################
sub PrepareClasslist {
    my %Sections;
    &clear_classlist_variables();
    #
    # Retrieve the classlist
    my $cid  = $ENV{'request.course.id'};
    my $cdom = $ENV{'course.'.$cid.'.domain'};
    my $cnum = $ENV{'course.'.$cid.'.num'};
    my ($classlist,$field_names) = &Apache::loncoursedata::get_classlist($cid,
                                                                  $cdom,$cnum);
    if (exists($ENV{'form.Section'})) {
        if (ref($ENV{'form.Section'})) {
            @SelectedSections = @{$ENV{'form.Section'}};
        } elsif ($ENV{'form.Section'} !~ /^\s*$/) {
            @SelectedSections = ($ENV{'form.Section'});
        }
    }
    @SelectedSections = ('all') if (! @SelectedSections);
    foreach (@SelectedSections) {
        if ($_ eq 'all') {
            @SelectedSections = ('all');
        }
    }
    #
    # Deal with instructors with restricted section access
    if ($ENV{'request.course.sec'} !~ /^\s*$/) {
        @SelectedSections = ($ENV{'request.course.sec'});
    }
    #
    # Set up %StudentData
    @StudentDataOrder = qw/fullname username domain id section status comments/;
    foreach my $field (@StudentDataOrder) {
        $StudentData{$field}->{'title'} = &mt($field);
        $StudentData{$field}->{'base_width'} = length(&mt($field));
        $StudentData{$field}->{'width'} = 
                               $StudentData{$field}->{'base_width'};
    }
    #
    # get the status requested
    $enrollment_status = 'Active';
    $enrollment_status = $ENV{'form.Status'} if (exists($ENV{'form.Status'}));
    #
    # Process the classlist
    while (my ($student,$student_data) = each (%$classlist)) {
        my $studenthash = ();
        for (my $i=0; $i< scalar(@$field_names);$i++) {
            my $field = $field_names->[$i];
            # Store the data
            $studenthash->{$field}=$student_data->[$i];
            # Keep track of the width of the fields
            next if (! exists($StudentData{$field}));
            my $length = length($student_data->[$i]);
            if ($StudentData{$field}->{'width'} < $length) {
                $StudentData{$field}->{'width'} = $length; 
            }
        }
        push (@FullClasslist,$studenthash);
        #
        # Build up a list of sections
        my $section = $studenthash->{'section'};
        if (! defined($section) || $section =~/^\s*$/ || $section == -1) {
            $studenthash->{'section'} = 'none';
            $section = $studenthash->{'section'};
        }
        $Sections{$section}++;
        #
        # Only put in the list those students we are interested in
        foreach my $sect (@SelectedSections) {
            if ( (($sect eq 'all') || 
                  ($section eq $sect)) &&
                 (($studenthash->{'status'} eq $enrollment_status) || 
                  ($enrollment_status eq 'Any')) 
                 ){
                push (@Students,$studenthash);
                last;
            }
        }
    }
    #
    # Put the consolidated section data in the right place
    if ($ENV{'request.course.sec'} !~ /^\s*$/) {
        @Sections = ($ENV{'request.course.sec'});
    } else {
        @Sections = sort {$a cmp $b} keys(%Sections);
        unshift(@Sections,'all'); # Put 'all' at the front of the list
    }
    #
    # Sort the Students
    my $sortby = 'fullname';
    $sortby = $ENV{'form.sort'} if (exists($ENV{'form.sort'}));
    my @TmpStudents = sort { $a->{$sortby} cmp $b->{$sortby} ||
                             $a->{'fullname'} cmp $b->{'fullname'} } @Students;
    @Students = @TmpStudents;
    # 
    # Now deal with that current student thing....
    $curr_student = undef;
    if (exists($ENV{'form.SelectedStudent'})) {
        my ($current_uname,$current_dom) = 
            split(':',$ENV{'form.SelectedStudent'});
        my $i;
        for ($i = 0; $i<=$#Students; $i++) {
            next if (($Students[$i]->{'username'} ne $current_uname) || 
                     ($Students[$i]->{'domain'}   ne $current_dom));
            $curr_student = $Students[$i];
            last; # If we get here, we have our student.
        }
        if (defined($curr_student)) {
            if ($i == 0) {
                $prev_student = undef;
            } else {
                $prev_student = $Students[$i-1];
            }
            if ($i == $#Students) {
                $next_student = undef;
            } else {
                $next_student = $Students[$i+1];
            }
        }
    }
    #
    if (exists($ENV{'form.StudentData'})) {
        if (ref($ENV{'form.StudentData'}) eq 'ARRAY') {
            @SelectedStudentData = @{$ENV{'form.StudentData'}};
        } else {
            @SelectedStudentData = ($ENV{'form.StudentData'});
        }
    } else {
        @SelectedStudentData = ('username');
    }
    foreach (@SelectedStudentData) {
        if ($_ eq 'all') {
            @SelectedStudentData = ('all');
            last;
        }
    }
    #
    return;
}


#######################################################
#######################################################

=pod

=item get_students

Returns a list of the selected students

=cut

#######################################################
#######################################################
sub get_students {
    if (! @Students) {
        &PrepareClasslist()
    }
    return @Students;
}

#######################################################
#######################################################

=pod

=item &current_student()

Returns a pointer to a hash containing data about the currently
selected student.

=cut

#######################################################
#######################################################
sub current_student { 
    return $curr_student;
}

#######################################################
#######################################################

=pod

=item &previous_student()

Returns a pointer to a hash containing data about the student prior
in the list of students.  Or something.  

=cut

#######################################################
#######################################################
sub previous_student { 
    return $prev_student;
}

#######################################################
#######################################################

=pod

=item &next_student()

Returns a pointer to a hash containing data about the next student
to be viewed.

=cut

#######################################################
#######################################################
sub next_student { 
    return $next_student;
}

#######################################################
#######################################################

=pod

=item &clear_sequence_variables()

=cut

#######################################################
#######################################################
sub clear_sequence_variables {
    undef($top_map);
    undef(@Sequences);
    undef(@Assessments);
}

#######################################################
#######################################################

=pod

=item &SetSelectedMaps($elementname)

Sets the @SelectedMaps array from $ENV{'form.'.$elementname};

=cut

#######################################################
#######################################################
sub SetSelectedMaps {
    my $elementname = shift;
    if (exists($ENV{'form.'.$elementname})) {
        if (ref($ENV{'form.'.$elementname})) {
            @SelectedMaps = @{$ENV{'form.'.$elementname}};
        } else {
            @SelectedMaps = ($ENV{'form.'.$elementname});
        }
    } else {
        @SelectedMaps = ('all');
    }
}


#######################################################
#######################################################

=pod

=item &Sequences_with_Assess()

Returns an array containing the subset of @Sequences which contain
assessments.

=cut

#######################################################
#######################################################
sub Sequences_with_Assess {
    my ($mode) = @_;
    $mode = 'selected' if (! defined($mode));
    my @Sequences_to_Show;
    foreach my $sequence (@Sequences) {
        next if ($sequence->{'num_assess'} < 1);
        if ($mode eq 'all') {
            push (@Sequences_to_Show,$sequence);
        } elsif ($mode eq 'selected') {
            foreach my $map_symb (@SelectedMaps) {
                if ($sequence->{'symb'} eq $map_symb || $map_symb eq 'all'){
                    push (@Sequences_to_Show,$sequence);
                    last; # Only put it in once
                }
            }
        }

    }
    return @Sequences_to_Show;
}

#######################################################
#######################################################

=pod

=item &PrepareCourseData($r)

=cut

#######################################################
#######################################################
sub PrepareCourseData {
    my ($r) = @_;
    &clear_sequence_variables();
    my ($top,$sequences,$assessments) = 
        &Apache::loncoursedata::get_sequence_assessment_data();
    if (! defined($top) || ! ref($top)) {
        # There has been an error, better report it
        &Apache::lonnet::logthis('top is undefined');
        return;
    }
    $top_map = $top if (ref($top));
    @Sequences = @{$sequences} if (ref($sequences) eq 'ARRAY');
    @Assessments = @{$assessments} if (ref($assessments) eq 'ARRAY');
    return;
}

#######################################################
#######################################################

=pod

=item &log_sequence($sequence,$recursive,$padding)

Write data about the sequence to a logfile.  If $recursive is not
undef the data is written recursively.  $padding is used for recursive
calls.

=cut

#######################################################
#######################################################
sub log_sequence {
    my ($seq,$recursive,$padding) = @_;
    $padding = '' if (! defined($padding));
    if (ref($seq) ne 'HASH') {
        &Apache::lonnet::logthis('log_sequence passed bad sequnce');
        return;
    }
    &Apache::lonnet::logthis($padding.'sequence '.$seq->{'title'});
    while (my($key,$value) = each(%$seq)) {
        next if ($key eq 'contents');
        if (ref($value) eq 'ARRAY') {
            for (my $i=0;$i< scalar(@$value);$i++) {
                &Apache::lonnet::logthis($padding.$key.'['.$i.']='.
                                         $value->[$i]);
            }
        } else {
            &Apache::lonnet::logthis($padding.$key.'='.$value);
        }
    }
    if (defined($recursive)) {
        &Apache::lonnet::logthis($padding.'-'x20);
        &Apache::lonnet::logthis($padding.'contains:');
        foreach my $item (@{$seq->{'contents'}}) {
            if ($item->{'type'} eq 'container') {
                &log_sequence($item,$recursive,$padding.'    ');
            } else {
                &Apache::lonnet::logthis($padding.'title = '.$item->{'title'});
                while (my($key,$value) = each(%$item)) {
                    next if ($key eq 'title');
                    if (ref($value) eq 'ARRAY') {
                        for (my $i=0;$i< scalar(@$value);$i++) {
                            &Apache::lonnet::logthis($padding.$key.'['.$i.']='.
                                                     $value->[$i]);
                        }
                    } else {
                        &Apache::lonnet::logthis($padding.$key.'='.$value);
                    }
                }
            }
        }
        &Apache::lonnet::logthis($padding.'end contents of '.$seq->{'title'});
        &Apache::lonnet::logthis($padding.'-'x20);
    }
    return;
}

##############################################
##############################################

=pod 

=item &StudentDataSelect($elementname,$status,$numvisible,$selected)

Returns html for a selection box allowing the user to choose one (or more) 
of the fields of student data available (fullname, username, id, section, etc)

=over 4

=item $elementname The name of the HTML form element

=item $status 'multiple' or 'single' selection box

=item $numvisible The number of options to be visible

=back

=cut

##############################################
##############################################
sub StudentDataSelect {
    my ($elementname,$status,$numvisible)=@_;
    if ($numvisible < 1) {
        return;
    }
    #
    # Build the form element
    my $Str = "\n";
    $Str .= '<select name="'.$elementname.'" ';
    if ($status ne 'single') {
        $Str .= 'multiple="true" ';
    }
    $Str .= 'size="'.$numvisible.'" >'."\n";
    #
    # Deal with 'all'
    $Str .= '    <option value="all" ';
    foreach (@SelectedStudentData) {
        if ($_ eq 'all') {
            $Str .= 'selected ';
            last;
        }
    }
    $Str .= ">all</option>\n";
    #
    # Loop through the student data fields
    foreach my $item (@StudentDataOrder) {
        $Str .= '    <option value="'.$item.'" ';
        foreach (@SelectedStudentData) {
            if ($item eq $_ ) {
                $Str .= 'selected ';
                last;
            }
        }
        $Str .= '>'.$item."</option>\n";
    }
    $Str .= "</select>\n";
    return $Str;
}

##############################################
##############################################

=pod 

=item &MapSelect($elementname,$status,$numvisible,$restriction) 

Returns html for a selection box allowing the user to choose one (or more) 
of the sequences in the course.  The values of the sequences are the symbs.
If the top sequence is selected, the value 'top' will result.

=over 4

=item $elementname The name of the HTML form element

=item $status 'multiple' or 'single' selection box

=item $numvisible The number of options to be visible

=item $restriction Code reference to subroutine which returns true or 
false.  The code must expect a reference to a sequence data structure.

=back

=cut

##############################################
##############################################
sub MapSelect {
    my ($elementname,$status,$numvisible,$restriction)=@_;
    if ($numvisible < 1) {
        return;
    }
    #
    # Set up array of selected items
    &SetSelectedMaps($elementname);
    #
    # Set up the restriction call
    if (! defined($restriction)) {
        $restriction = sub { 1; };
    }
    #
    # Build the form element
    my $Str = "\n";
    $Str .= '<select name="'.$elementname.'" ';
    if ($status ne 'single') {
        $Str .= 'multiple="true" ';
    }
    $Str .= 'size="'.$numvisible.'" >'."\n";
    #
    # Deal with 'all'
    foreach (@SelectedMaps) {
        if ($_ eq 'all') {
            @SelectedMaps = ('all');
            last;
        }
    }
    #
    # Put in option for 'all'
    $Str .= '    <option value="all" ';
    foreach (@SelectedMaps) {
        if ($_ eq 'all') {
            $Str .= 'selected ';
            last;
        }
    }
    $Str .= ">all</option>\n";
    #
    # Loop through the sequences
    foreach my $seq (@Sequences) {
        next if (! $restriction->($seq));
        $Str .= '    <option value="'.$seq->{'symb'}.'" ';
        foreach (@SelectedMaps) {
            if ($seq->{'symb'} eq $_) {
                $Str .= 'selected ';
                last;
            }
        }
        $Str .= '>'.$seq->{'title'}."</option>\n";
    }
    $Str .= "</select>\n";
    return $Str;
}

##############################################
##############################################

=pod 

=item &SectionSelect($elementname,$status,$numvisible) 

Returns html for a selection box allowing the user to choose one (or more) 
of the sections in the course.  

Uses the package variables @Sections and @SelectedSections
=over 4

=item $elementname The name of the HTML form element

=item $status 'multiple' or 'single' selection box

=item $numvisible The number of options to be visible

=back

=cut

##############################################
##############################################
sub SectionSelect {
    my ($elementname,$status,$numvisible)=@_;
    if ($numvisible < 1) {
        return;
    }
    #
    # Make sure we have the data we need to continue
    if (! @Sections) {
        &PrepareClasslist()
    }
    #
    # Build the form element
    my $Str = "\n";
    $Str .= '<select name="'.$elementname.'" ';
    if ($status ne 'single') {
        $Str .= 'multiple="true" ';
    }
    $Str .= 'size="'.$numvisible.'" >'."\n";
    #
    # Loop through the sequences
    foreach my $s (@Sections) {
        $Str .= '    <option value="'.$s.'" ';
        foreach (@SelectedSections) {
            if ($s eq $_) {
                $Str .= 'selected ';
                last;
            }
        }
        $Str .= '>'.$s."</option>\n";
    }
    $Str .= "</select>\n";
    return $Str;
}

#######################################################
#######################################################

=pod

=item &CreateAndParseOutputSelector()

Construct a selection list of options for output and parse output selections.

=cut

#######################################################
#######################################################
sub OutputDescriptions {
    my (@OutputOptions) = @_;
    my $Str = '';
    $Str .= "<h2>Output Modes</h2>\n";
    $Str .= "<dl>\n";
    foreach my $outputmode (@OutputOptions) {
	$Str .="    <dt>".$outputmode->{'name'}."</dt>\n";
	$Str .="        <dd>".$outputmode->{'description'}."</dd>\n";
    }
    $Str .= "</dl>\n";
    return $Str;
}

sub CreateAndParseOutputSelector {
    my ($elementname,$default,@OutputOptions) = @_;
    my $output_mode;
    my $show;
    my $Str = '';
    #
    # Format for output options is 'mode, restrictions';
    my $selected = $default;
    if (exists($ENV{'form.'.$elementname})) {
        if (ref($ENV{'form.'.$elementname} eq 'ARRAY')) {
            $selected = $ENV{'form.'.$elementname}->[0];
        } else {
            $selected = $ENV{'form.'.$elementname};
        }
    }
    #
    # Set package variables describing output mode
    $output_mode = 'html';
    $show        = 'all';
    foreach my $option (@OutputOptions) {
        next if ($option->{'value'} ne $selected);
        $output_mode = $option->{'mode'};
        $show        = $option->{'show'};
    }
    #
    # Build the form element
    $Str = qq/<select size="5" name="$elementname">/;
    foreach my $option (@OutputOptions) {
        if (exists($option->{'special'}) && 
            $option->{'special'} =~ /do not show/) {
            next;
        }
        $Str .= "\n".'    <option value="'.$option->{'value'}.'"';
        $Str .= " selected " if ($option->{'value'} eq $selected);
        $Str .= ">".&mt($option->{'name'})."<\/option>";
    }
    $Str .= "\n</select>";
    return ($Str,$output_mode,$show);
}

###############################################
###############################################

=pod 

=item &Gather_Student_Data()

Ensures all student data is up to date.

=cut

###############################################
###############################################
sub Gather_Student_Data {
    my ($r) = @_;
    my $c = $r->connection();
    #
    &Apache::loncoursedata::clear_internal_caches();
    #
    my @Sequences = &Apache::lonstatistics::Sequences_with_Assess();
    #
    my @Students = @Apache::lonstatistics::Students;
    #
    # Open the progress window
    my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
        ($r,'Statistics Compilation Status',
         'Statistics Compilation Progress', scalar(@Students));
    #
    while (my $student = shift @Students) {
        return if ($c->aborted());
        my ($status,undef) = &Apache::loncoursedata::ensure_current_data
            ($student->{'username'},$student->{'domain'},
             $ENV{'request.course.id'});
        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
                                                 &mt('last student'));
    }
    &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    $r->rflush();
}

###############################################
###############################################

=pod 

=item &Gather_Full_Student_Data()

Ensures all student data is up to date.

=cut

###############################################
###############################################
sub Gather_Full_Student_Data {
    my ($r,$formname,$inputname) = @_;
    my $status_type;
    if (defined($formname)) {
        $status_type = 'inline';
    } else {
        $status_type = 'popup';
    }
    my $c = $r->connection();
    #
    &Apache::loncoursedata::clear_internal_caches();
    #
    my @Students = @Apache::lonstatistics::Students;
    #
    # Open the progress window
    my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
        ($r,&mt('Student Data Compilation Status'),
         &mt('Student Data Compilation Progress'), scalar(@Students),
         $status_type,undef,$formname,$inputname);
    #
    while (my $student = shift @Students) {
        return if ($c->aborted());
        my ($status,undef) = &Apache::loncoursedata::ensure_current_full_data
            ($student->{'username'},$student->{'domain'},
             $ENV{'request.course.id'});
        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
                                                 &mt('last student'));
    }
    &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    $r->rflush();
}

##################################################
##################################################
sub DisplayClasslist {
    my ($r)=@_;
    &Apache::lonhtmlcommon::add_breadcrumb
        ({text=>'Select One Student'});
    #
    # Output some of the standard interface components
    my $Str;
    $Str .= &Apache::lonhtmlcommon::breadcrumbs(undef,'Select One Student');
    $Str .= '<p><table cellspacing="5">'."\n";
    $Str .= '<tr>';
    $Str .= '<th align="center"><b>'.&mt('Sections').'</b></th>';
    $Str .= '<th align="center"><b>'.&mt('Enrollment Status').'</b></th>';
    $Str .= '</tr>'.$/;
    $Str .= '<tr>';
    $Str .= '<td>'.
        &Apache::lonstatistics::SectionSelect('Section','multiple',5).
        '</td>';
    $Str .= '<td>'.
        &Apache::lonhtmlcommon::StatusOptions(undef,undef,5).
        '</td>';
    
    $Str .= '</tr>'.$/;
    $Str .= '</table></p>';
    $Str .= '<input type="submit" name="selectstudent" value="'.
        &mt('Update Display').'" />';
    $r->print($Str);
    $r->rflush();
    #
    my @Fields = ('fullname','username','domain','id','section','status');
    #
    $Str = '';
    if (! @Students) {
        if ($SelectedSections[0] eq 'all') { 
            if (lc($ENV{'form.Status'}) eq 'any') {
                $Str .= '<h2>'.
                    &mt('There are no students in the course.').
                    '</h2>';
            } elsif (lc($ENV{'form.Status'}) eq 'active') {
                $Str .= '<h2>'.
                &mt('There are no currently enrolled students in the course.').
                    '</h2>';
            } elsif (lc($ENV{'form.Status'}) eq 'expired') {
                $Str .= '<h2>'.
                    &mt('There are no previously enrolled students in the course.').
                        '</h2>';
            }
        } else { 
            my $sections;
            if (lc($ENV{'form.Status'}) eq 'any') {
                $Str .= '<h2>'.
                    &mt('There are no students in the selected sections.').
                    '</h2>';
            } elsif (lc($ENV{'form.Status'}) eq 'active') {
                $Str .= '<h2>'.
                    &mt('There are no currently enrolled students in the selected sections.').
                    '</h2>';
            } elsif (lc($ENV{'form.Status'}) eq 'expired') {
                $Str .= '<h2>'.
                    &mt('There are no previously enrolled students in the selected sections.').
                    '</h2>';
            }
        }
        $Str.= '<a href="/adm/statistics?reportSelected=student_assessment">'.
            &mt('Click here to return to the chart').'</a>';
        $r->print($Str);
        $r->rflush();
        return;
    }

    # "Click" is asinine but it is probably not my place to change the world.
    $Str .= '<h2>Click on a students name or username to view their chart</h2>';
    $Str .= '<table border="0"><tr><td bgcolor="#777777">'."\n";
    $Str .= '<table border="0" cellpadding="3"><tr bgcolor="#e6ffff">'."\n";
    foreach my $field (@Fields) {
        $Str .= '<th><a href="/adm/statistics?'.
            'reportSelected=student_assessment&'.
            'selectstudent=1&'.
            'sort='.$field.'">'.&mt($field).
            '</a></th>';
    }
    $Str .= '</tr>'."\n";
    #
    my $alternate = 0;
    foreach my $student (@Students) { # @Students is a package variable
        my $sname = $student->{'username'}.':'.$student->{'domain'};
        if($alternate) {
            $Str .= '<tr bgcolor="#ffffe6">';
        } else {
            $Str .= '<tr bgcolor="#ffffc6">';
        }
        $alternate = ($alternate + 1) % 2;
        #
        foreach my $field (@Fields) {
            $Str .= '<td>';
            if ($field eq 'fullname' || $field eq 'username') {
                $Str .= '<a href="/adm/statistics?reportSelected=';
                $Str .= &Apache::lonnet::escape('student_assessment');
                $Str .= '&sort='.&Apache::lonnet::escape($ENV{'form.sort'});
                $Str .= '&SelectedStudent=';
                $Str .= &Apache::lonnet::escape($sname).'">';
                $Str .= $student->{$field}.'&nbsp';
                $Str .= '</a>';
            } elsif ($field eq 'status') {
                $Str .= &mt($student->{$field});
            } else {
                $Str .= $student->{$field};
            }
            $Str .= '</td>';
        }
        $Str .= "</tr>\n";
    }
    $Str .= '</table></td></tr></table>'."\n";
    #
    $r->print($Str);
    $r->rflush();
    #
    return;
}

##############################################
##############################################
sub CreateMainMenu {
    #
    # Define menu data
    my @reports = ({ internal_name => 'problem_statistics',
                     name => &mt('Overall Problem Statistics'),
                     short_description => 
    &mt('Student performance statistics on all problems.'),
                 },
                   { internal_name => 'problem_analysis',
                     name => &mt('Detailed Problem Analysis'),
                     short_description => 
    &mt('Detailed statistics and graphs of student performance on problems.'),
                 },
                   { internal_name => 'submissiontime_analysis',
                     name => &mt('Submission Time Plots'),
                     short_description => 
    &mt('Display and analysis of submission times on assessments.'),
                 },
                   { internal_name => 'student_submission_reports',
                     name => &mt('Student Submission Reports'),
                     short_description => 
    &mt('Prepare reports of student submissions.'),
                 },
                   { internal_name => 'survey_reports',
                     name => &mt('Survey Reports'),
                     short_description => 
    &mt('Prepare reports on survey results.'),
                 },
                   { internal_name => 'correct_problems_plot',
                     name => &mt('Correct Problems Plot'),
                     short_description => 
    &mt('Display a histogram of student performance in the course.'),
                 },
#                   { internal_name => 'student_assessment',
#                     name => &mt('Problem Status Chart'),
#                     short_description => 
#    &mt('Brief view of each students performance in course.'),
#                 },
                   # 'percentage'  => 'Correct-problems Plot',
                   # 'activitylog' => 'Activity Log',
                   );
    #
    # Create the menu
    my $Str;
    $Str .= '<h2>'.&mt('Please select a report to generate').'</h2>';
    foreach my $reportdata (@reports) {
        $Str .='    <h3><a href="/adm/statistics?reportSelected='.
            $reportdata->{'internal_name'}.'" >'.
            $reportdata->{'name'}."</a></h3>\n";
        $Str .= '    '.('&nbsp;'x8).$reportdata->{'short_description'}.
            "\n";
    }
    $Str .="</dl>\n";
    #
    return $Str;
}

##############################################
##############################################
sub handler {
    my $r=shift;
    my $c = $r->connection();
    #
    # Check for overloading
    my $loaderror=&Apache::lonnet::overloaderror($r);
    if ($loaderror) { return $loaderror; }
    $loaderror=
       &Apache::lonnet::overloaderror($r,
         $ENV{'course.'.$ENV{'request.course.id'}.'.home'});
    if ($loaderror) { return $loaderror; }
    #
    # Check for access
    if (! &Apache::lonnet::allowed('vgr',$ENV{'request.course.id'})) {
        $ENV{'user.error.msg'}=
            $r->uri.":vgr:0:0:Cannot view grades for complete course";
        if (! &Apache::lonnet::allowed('vgr',
                      $ENV{'request.course.id'}.'/'.$ENV{'request.course.sec'})) {
            $ENV{'user.error.msg'}=
                $r->uri.":vgr:0:0:Cannot view grades with given role";
            return HTTP_NOT_ACCEPTABLE;
        }
    }
    #
    # Send the header
    &Apache::loncommon::no_cache($r);
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    if ($r->header_only) { return OK; }
    #
    # Extract form elements from query string
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
                                            ['sort','reportSelected',
                                             'SelectedStudent']);
    #
    # Give the LON-CAPA page header
    my $style = <<ENDSTYLE;
<style type="text/css">
    ul.sub_studentans { list-style-type: none }
    ul.sub_correctans { list-style-type: none }
    tr.even           { background-color: \#CCCCCC }
    td.essay          { border: 1px solid gray; }
</style>
ENDSTYLE
    $r->print('<html><head><title>'.
              &mt('Course Statistics and Charts').
              '</title>'.$style.
              "</head>\n".
              &Apache::loncommon::bodytag('Course Statistics and Charts'));
    $r->rflush();
    # 
    # Either print out a menu for them or send them to a report
    &Apache::lonhtmlcommon::clear_breadcrumbs();
    &Apache::lonhtmlcommon::add_breadcrumb({href=>'/adm/statistics',
                                            title=>'Statistics',
                                            text =>'Statistics',
                                            faq=>139,
                                            bug=>'Statistics and Charts'});
    if (! exists($ENV{'form.reportSelected'}) || 
        $ENV{'form.reportSelected'} eq '') {
        $r->print(&Apache::lonhtmlcommon::breadcrumbs
                  (undef,&mt('Statistics Main Page')).
                  &CreateMainMenu());
    } else {
    #
        if (! &Apache::lonmysql::verify_sql_connection()) {
            my $serveradmin = $r->dir_config('lonAdmEMail');
            $r->print('<h2><font color="Red">'.
                      &mt('Unable to connect to database!').
                      '</font></h2>');
            $r->print('<p>'.
                      &mt('Please notify the server administrator ').
                      '<b>'.$serveradmin.'</b></p>');
            $r->print('<p>'.
                      &mt('Course Statistics and Charts cannot be '.
                          'retrieved until the database is restarted.  '.
                          'Your data is intact but cannot be displayed '.
                          'at this time.').'</p>');
            $r->print('</body></html>');
            return;
        }
        #
        # Clean out the caches
        if (exists($ENV{'form.ClearCache'})) {
            &Apache::loncoursedata::delete_caches($ENV{'requres.course.id'});
        }
        #
        # Begin form output
        $r->print('<form name="Statistics" ');
        $r->print('method="post" action="/adm/statistics">');
        $r->rflush();
        #
        my $GoToPage = $ENV{'form.reportSelected'};
        #
        # Set up the statistics and chart environment
        &PrepareCourseData($r);
        #
        $r->print('<input type="hidden" name="reportSelected" value="'.
                  $GoToPage.'">');
        if($GoToPage eq 'activitylog') {
#        &Apache::lonproblemstatistics::Activity();
        } elsif($GoToPage eq 'problem_statistics') {
            &Apache::lonhtmlcommon::add_breadcrumb
                ({href=>'/adm/statistics?reportselected=problem_statistics',
                  text=>'Overall Problem Statistics'});
            &Apache::lonproblemstatistics::BuildProblemStatisticsPage($r,$c);
        } elsif($GoToPage eq 'problem_analysis') {
            &Apache::lonhtmlcommon::add_breadcrumb
                ({href=>'/adm/statistics?reportselected=problem_analysis',
                  text=>'Detailed Problem Analysis'});
            &Apache::lonproblemanalysis::BuildProblemAnalysisPage($r,$c);
        } elsif($GoToPage eq 'submissiontime_analysis') {
            &Apache::lonhtmlcommon::add_breadcrumb
                ({href=>
                      '/adm/statistics?reportselected=submissiontime_analysis',
                      text=>'Submission Time Plots'});
            &Apache::lonsubmissiontimeanalysis::BuildSubmissionTimePage($r,$c);
        } elsif($GoToPage eq 'student_submission_reports') {
            &Apache::lonhtmlcommon::add_breadcrumb
                ({href=>
                  '/adm/statistics?reportselected=student_submission_reports',
                  text=>'Student Submission Reports'});
            &Apache::lonstudentsubmissions::BuildStudentSubmissionsPage($r,$c);
        } elsif($GoToPage eq 'survey_reports') {
            &Apache::lonhtmlcommon::add_breadcrumb
                ({href=>
                  '/adm/statistics?reportselected=survey_reports',
                  text=>'Survey Reports'});
            &Apache::lonsurveyreports::BuildSurveyReportsPage($r,$c);
        } elsif($GoToPage eq 'correct_problems_plot') {
            &Apache::lonhtmlcommon::add_breadcrumb
                ({href=>'/adm/statistics?reportselected=correct_problems_plot',
                  text=>'Correct Problems Plot'});
            &Apache::loncorrectproblemplot::BuildCorrectProblemsPage($r,$c);
        } elsif($GoToPage eq 'student_assessment') {
            &Apache::lonhtmlcommon::clear_breadcrumbs();
            &Apache::lonhtmlcommon::add_breadcrumb
                ({href=>'/adm/statistics?reportselected=student_assessment',
                  text=>'Chart'});
            &Apache::lonstudentassessment::BuildStudentAssessmentPage($r,$c);
        }
        #
        $r->print("</form>\n");
    }
    $r->print("</body>\n</html>\n");
    $r->rflush();
    #
    return OK;
}

1;

#######################################################
#######################################################

=pod

=back

=cut

#######################################################
#######################################################

__END__

