# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or 
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::portfolio;
use strict;
use Apache::Constants qw(:common :http);
use Apache::loncommon;
use Apache::lonnet;
use Apache::lontexconvert;
use Apache::lonfeedback;
use Apache::lonlocal;

# receives a file name and path stub from username/userfiles/portfolio/
# returns an anchor tag consisting encoding filename and currentpath
sub make_anchor {
    my ($filename, $current_path, $current_mode, $field_name) = @_;
    my $anchor = '<a href="/adm/portfolio?selectfile='.$filename.'&currentpath='.$current_path.'&mode='.$current_mode.'&fieldname='.$field_name.'">'.$filename.'</a>';
    return $anchor;
}
my $dirptr=16384;
sub display_common {
    my ($r,$current_path,$is_empty,$dir_list)=@_;
    my $iconpath= $r->dir_config('lonIconsURL') . "/";
    $r->print('<table border="0" cellspacing="2" cellpadding="2"><tr valign="middle">');
    $r->print('<td bgcolor="#ccddaa" align="center">');
    my $displayOut = '<form method="post" enctype="multipart/form-data">';
    $displayOut .= '<input name="uploaddoc" type="file" />'.
        '<input type="hidden" name="currentpath" value="'.$current_path.'" />'.
        '<input type="hidden" name="action" value="'.$ENV{"form.action"}.'" />'.
        '<input type="hidden" name="fieldname" value="'.$ENV{"form.fieldname"}.'" />'.
        '<input type="hidden" name="mode" value="'.$ENV{"form.mode"}.'" />'. 
        '<input type="submit" name="storeupl" value="Upload" />'.
        '</form>';
    $r->print($displayOut);
    $r->print('</td></tr><tr><td bgcolor="#ccddaa" align="center">');
    $displayOut = '<form method="post">';
    $displayOut .= '<input name="newdir" type="input" />'.
        '<input type="hidden" name="currentpath" value="'.$current_path.'" />'.
        '<input type="hidden" name="action" value="'.$ENV{"form.action"}.'" />'.
        '<input type="hidden" name="fieldname" value="'.$ENV{"form.fieldname"}.'" />'.
        '<input type="hidden" name="mode" value="'.$ENV{"form.mode"}.'" />'. 
        '<input type="submit" name="createdir" value="'.&mt("Create Directory").'" />'.
        '</form>';
    $r->print($displayOut);
    $r->print('</td></tr></table>');
    my @tree = split (/\//,$current_path);
    $r->print('<font size="+2">'.&make_anchor('portfolio','/',$ENV{"form.mode"},$ENV{"form.fieldname"}).'/');
    if (@tree > 1){
        my $newCurrentPath = '';
        for (my $i = 1; $i< @tree; $i++){
            $newCurrentPath .= $tree[$i].'/';
            $r->print(&make_anchor($tree[$i],'/'.$newCurrentPath, $ENV{"form.mode"},$ENV{"form.fieldname"}).'/');
        }
    }
    $r->print('</font>');
    &Apache::lonhtmlcommon::store_recent('portfolio',$current_path,$current_path);
    $r->print('<br /><form method=post action="/adm/portfolio?mode='.$ENV{"form.mode"}.'&fieldname='.$ENV{"form.fieldname"}.'">'.
	      &Apache::lonhtmlcommon::select_recent('portfolio','currentpath',
						    'this.form.submit();'));
    $r->print("</form>");
}
sub display_directory {
    my ($r,$current_path,$is_empty,$dir_list)=@_;
    my $iconpath= $r->dir_config('lonIconsURL') . "/";
    my $display_out;
    my %locked_files = &Apache::lonnet::get_marked_as_readonly_hash ($ENV{'user.domain'},$ENV{'user.name'});
    if ($is_empty && ($current_path ne '/')) {
        $display_out = '<form method="post" action="/adm/portfolio">'.
        '<input type="hidden" name="action" value="deletedir" />'.
        '<input type="submit" name="deletedir" value="'.&mt("Delete Directory").'" />'.
        '<input type="hidden" name="selectfile" value="" />'.
        '<input type="hidden" name="currentpath" value="'.$current_path.'" />'.
        '</form>';
        
        $r->print($display_out);
	return;
    }
    $r->print('<table border="0" cellspacing="2" cellpadding="2">'.
            '<tr><th>Actions</th><th>&nbsp;</th><th>Name</th><th>Size</th><th>Last Modified</th></tr>');
    my $href_location="/uploaded/$ENV{'user.domain'}/$ENV{'user.name'}/portfolio$current_path";
    $r->print('<form method="post" action="/adm/portfolio">');
    foreach my $line (sort 
		      { 
			  my ($afile)=split('&',$a,2);
			  my ($bfile)=split('&',$b,2);
			  return (lc($afile) cmp lc($bfile));
		      } (@$dir_list)) {
    	#$strip holds directory/file name
    	#$dom 
    	my ($filename,$dom,undef,$testdir,undef,undef,undef,undef,$size,undef,$mtime,undef,undef,undef,$obs,undef)=split(/\&/,$line,16); 
    	if (($filename ne '.') && ($filename ne '..')) {
            if ($dirptr&$testdir) {
                $r->print('<tr bgcolor="#FFAA99"><td><img src="'.$iconpath.'folder_closed.gif"></td>');
                $r->print('<td>Go to ...</td>');
                $r->print('<td>'.&make_anchor($filename.'/',$current_path.$filename.'/',$ENV{'form.mode'},$ENV{"form.fieldname"}).'</td>'); 
                $r->print('</tr>'); 
            } else {
                $r->print('<tr bgcolor="#CCCCFF">');
                if (exists $locked_files{$current_path.$filename}){
                    $r->print('<td>Locked</td>');
                } else {
                    $r->print('<td>Delete<input type="checkbox" name="selectfile" value="'.$filename.'" />
                        <a href="/adm/portfolio?rename='.$filename.'&amp;currentpath='.$current_path.'">Rename</a>                        
                        </td>');
                }
                $r->print('<td><img src="'.$iconpath.'unknown.gif"></td>');
                $r->print('<td><a href="'.$href_location.$filename.'">'.
			    $filename.'</a></td>'); 
                $r->print('<td>'.$size.'</td>');
                $r->print('<td>'.&Apache::lonlocal::locallocaltime($mtime).'</td>');
                $r->print('</tr>'); 
            }
        }
    }
#   <tr bgcolor="#FFAA99"> pink bg 
#   <tr bgcolor="#CCCCFF"> blue bg            
#   $r->print(&display_directory($current_path, $currentFile, @dir_list));
#    $r->print('</td>><td>');
#   $r->print(&display_actions($current_path, $currentFile, $isEmpty));
    $r->print('</table>
    <input type="submit" name="doit" value="Delete Checked Files" />
    <input type="hidden" name="action" value="delete" />
    <input type="hidden" name="currentpath" value="'.$current_path.'" />
    </form>');
}
sub display_file_select {
    my ($r,$current_path,$is_empty,$dir_list)=@_;
    my $iconpath= $r->dir_config('lonIconsURL') . "/";
    my $display_out;
    my $checked_files = &Apache::lonnet::files_in_path($ENV{'user.name'}, $ENV{'form.currentpath'});
    if ($is_empty && ($current_path ne '/')) {
        $display_out = '<form method="post" action="/adm/portfolio">'.
        '<input type="hidden" name="action" value="deletedir" />'.
        '<input type="submit" name="deletedir" value="'.&mt("Delete Directory").'" />'.
        '<input type="hidden" name="selectfile" value="" />'.
        '<input type="hidden" name="currentpath" value="'.$current_path.'" />'.

        '</form>';
        
        $r->print($display_out);
	return;
    }
    $r->print('<table border="0" cellspacing="2" cellpadding="2">'.
            '<tr><th>Select</th><th>&nbsp;</th><th>Name</th><th>Size</th><th>Last Modified</th></tr>');
    my $href_location="/uploaded/$ENV{'user.domain'}/$ENV{'user.name'}/portfolio$current_path";
    $r->print('<form method="post" name="checkselect" action="/adm/portfolio">');
    foreach my $line (sort 
		      { 
			  my ($afile)=split('&',$a,2);
			  my ($bfile)=split('&',$b,2);
			  return (lc($afile) cmp lc($bfile));
		      } (@$dir_list)) {
    	#$strip holds directory/file name
    	#$dom 
    	my ($filename,$dom,undef,$testdir,undef,undef,undef,undef,$size,undef,$mtime,undef,undef,undef,$obs,undef)=split(/\&/,$line,16); 
    	$filename =~ s/\s+$//;
    	if (($filename ne '.') && ($filename ne '..')) {
            if ($dirptr&$testdir) {
                $r->print('<tr bgcolor="#FFAA99"><td><img src="'.$iconpath.'folder_closed.gif"></td>');
                $r->print('<td>Go to ...</td>');
                $r->print('<td>'.&make_anchor($filename.'/',$current_path.$filename.'/',$ENV{'form.mode'},$ENV{"form.fieldname"}).'</td>'); 
                $r->print('</tr>'); 
            } else {
                $r->print('<tr bgcolor="#CCCCFF">');
                $r->print('<td><input type="checkbox" name="selectfile" value="'.$filename.'"'); 
                if ($$checked_files{$filename} eq 'selected') {
                    $r->print("CHECKED");
                }
                $r->print('></td>');
                $r->print('<td><img src="'.$iconpath.'unknown.gif"></td>');
                $r->print('<td><a href="'.$href_location.$filename.'">'.
			    $filename.'</a></td>'); 
                $r->print('<td>'.$size.'</td>');
                $r->print('<td>'.&Apache::lonlocal::locallocaltime($mtime).'</td>');
                $r->print('</tr>'); 
            }
        }
    }
    $r->print('</table>
            <input type="hidden" name="continue" value="true">
            <input type="hidden" name="fieldname" value="'.$ENV{'form.fieldname'}.'">
            <input type="hidden" name="mode" value="selectfile">
            <input type="submit" name="submit" value="Select checked files, and continue selecting." /><br />
            <input type="button" name="doit" onClick= "finishSelect();" value="Select checked files, and close window" />
            <input type="hidden" name="currentpath" value="'.$current_path.'" />
    </form>');
}

sub open_form {
    my ($r)=@_;
    my @files=&Apache::loncommon::get_env_multiple('form.selectfile');
    $r->print('<form method="post" action="/adm/portfolio">');
    $r->print('<input type="hidden" name="action" value="'.
	      $ENV{'form.action'}.'" />');
    $r->print('<input type="hidden" name="confirmed" value="1" />');
    foreach (@files) {
        $r->print('<input type="hidden" name="selectfile" value="'.
	      $_.'" />');
    }
    $r->print('<input type="hidden" name="currentpath" value="'.
	      $ENV{'form.currentpath'}.'" />');
}

sub close_form {
    my ($r)=@_;
    $r->print('<p><input type="submit" value="'.&mt('Continue').
	      '" /></p></form>');
    $r->print('<form action="/adm/portfolio" method="POST">
               <p>
              <input type="hidden" name="currentpath" value="'.
	      $ENV{'form.currentpath'}.'" />
                 <input type="submit" value="'.&mt('Cancel').'" />
               </p></form>');

}

sub display_file {
    my ($path,$filename)=@_;
    my $display_file_text;
    if (!defined($path)) { $path=$ENV{'form.currentpath'}; }
    if (!defined($filename)) { 
        $filename=$ENV{'form.selectfile'};
        $display_file_text = '<tt>'.$path.$filename.'</tt>';
    } elsif (ref($filename) eq "ARRAY") {
        foreach (@$filename) {
            $display_file_text .= '<tt>'.$path.$_.'</tt><br />';
        }
    } elsif (ref($filename) eq "SCALAR") {
        $display_file_text = '<tt>'.$path.$filename.'</tt>';        
    }
    return $display_file_text;
}

sub done {
    return ('<h3><a href="/adm/portfolio?currentpath='.
	    $ENV{'form.currentpath'}.
	    '&fieldname='.$ENV{'form.fieldname'}.
	    '&mode='.$ENV{'form.mode'}.
	    '">'.&mt('Done').'</a></h3>');
}

sub delete {
    my ($r)=@_;
    my @check;
    my $file_name = $ENV{'form.currentpath'}.$ENV{'form.selectfile'};
    my @files=&Apache::loncommon::get_env_multiple('form.selectfile');

    if (&Apache::lonnet::is_locked($file_name,$ENV{'user.domain'},$ENV{'user.name'} ) eq 'true') {
        $r->print ("The file is locked and cannot be deleted.<br />");
        $r->print(&done());
    } else {
        if (scalar(@files)) {
            &open_form($r);
            $r->print('<p>'.&mt('Delete').' '.&display_file(undef,\@files).'?</p>');
            &close_form($r);
        } else {
            $r->print("No file was checked to delete.<br />");
            $r->print(&done());
        }
    }
} 

sub delete_confirmed {
    my ($r)=@_;
    my @files=&Apache::loncommon::get_env_multiple('form.selectfile');
    my $result;
    foreach my $delete_file (@files) {
        $result=&Apache::lonnet::removeuserfile($ENV{'user.name'},
					       $ENV{'user.domain'},'portfolio'.
					       $ENV{'form.currentpath'}.
					       $delete_file);
        if ($result ne 'ok') {
	$r->print('<font color="red"> An error occured ('.$result.
		  ') while trying to delete '.&display_file(undef, $delete_file).'</font><br />');
        }
    }
    $r->print(&done());
}

sub delete_dir {
    my ($r)=@_;
    &open_form($r);
    $r->print('<p>'.&mt('Delete').' '.&display_file().'?</p>');
    &close_form($r);
} 

sub delete_dir_confirmed {
    my ($r)=@_;
    my $directory_name = $ENV{'form.currentpath'};
    $directory_name =~ m/\/$/;
    $directory_name = $`;
    my $result=&Apache::lonnet::removeuserfile($ENV{'user.name'},
					       $ENV{'user.domain'},'portfolio'.
					       $directory_name);
					       
    if ($result ne 'ok') {
	$r->print('<font color="red"> An error occured (dir) ('.$result.
		  ') while trying to delete '.$directory_name.'</font><br />');
    } else {
        # now remove from recent
#        $r->print('<br /> removing '.$directory_name.'<br /');
        &Apache::lonhtmlcommon::remove_recent('portfolio',[$directory_name.'/']);
        my @dirs = split m!/!, $directory_name;
        
#        $directory_name =~ m/^(\/*\/)(\/*.)$/;
        $directory_name='/';
        for (my $i=1; $i < (@dirs - 1); $i ++){
            $directory_name .= $dirs[$i].'/';
        }
        $ENV{'form.currentpath'} = $directory_name;
    }
    $r->print(&done());
}

sub rename {
    my ($r)=@_;
    my $file_name = $ENV{'form.currentpath'}.$ENV{'form.rename'};
    if (&Apache::lonnet::is_locked($file_name,$ENV{'user.domain'},$ENV{'user.name'}) eq 'true') {
        $r->print ("The file is locked and cannot be renamed.<br />");
        $r->print(&done());
    } else {
        &open_form($r);
        $r->print('<p>'.&mt('Rename').' '.&display_file().' to 
                   <input name="filenewname" type="input" size="50" />?</p>');
        &close_form($r);
    }
}

sub rename_confirmed {
    my ($r)=@_;
    my $filenewname=&Apache::lonnet::clean_filename($ENV{'form.filenewname'});
    if ($filenewname eq '') {
	$r->print('<font color="red">'.
		  &mt("Error: no valid filename was provided to rename to.").
		  '</font><br />');
	$r->print(&done());
	return;
    } 
    my $result=
	&Apache::lonnet::renameuserfile($ENV{'user.name'},$ENV{'user.domain'},
            'portfolio'.$ENV{'form.currentpath'}.$ENV{'form.selectfile'},
            'portfolio'.$ENV{'form.currentpath'}.$filenewname);
    if ($result ne 'ok') {
	$r->print('<font color="red"> An errror occured ('.$result.
		  ') while trying to rename '.&display_file().' to '.
		  &display_file(undef,$filenewname).'</font><br />');
    }
    if ($filenewname ne $ENV{'form.filenewname'}) {
        $r->print("The new file name was changed from:<br /><strong>".$ENV{'form.filenewname'}."</strong> to <strong>$filenewname </strong>");
    }
    $r->print(&done());
}
sub select_files {
    my ($r)=@_;
    if ($ENV{'form.continue'} eq 'true') {
        # here we update the selections for the currentpath
        # eventually, have to handle removing those not checked, but . . . 
        my @items=&Apache::loncommon::get_env_multiple('form.selectfile');
        &Apache::lonnet::save_selected_files($ENV{'user.name'}, $ENV{'form.currentpath'}, @items);
    } else {
        if ($ENV{'form.currentpath'} eq '/') {
            #empty the file for a fresh start
            # &Apache::lonnet::clear_selected_files($ENV{'user.name'});
        }
    }
    my @files = &Apache::lonnet::files_not_in_path($ENV{'user.name'}, $ENV{'form.currentpath'});
    my $java_files = join ",", @files;
    if ($java_files) {
        $java_files.=',';
    }
    my $javascript =(<<ENDSMP);
        <script language='javascript'>
        function finishSelect() {
ENDSMP
    $javascript .= 'fileList = "'.$java_files.'";';
    $javascript .= (<<ENDSMP);
            for (i=0;i<document.forms.checkselect.length;i++) { 
                if (document.forms.checkselect[i].checked){
                    fileList = fileList + document.forms.checkselect.currentpath.value + document.forms.checkselect[i].value + "," ;
                }
            }
            opener.document.forms.lonhomework.
ENDSMP
    $javascript .= $ENV{'form.fieldname'};
    $javascript .= (<<ENDSMP);
        .value=fileList;
            self.close();
        }
        </script>
ENDSMP
    $r->print($javascript);
    $r->print("<h1>Select portfolio files</h1>
                Check as many as you wish in response to the essay problem.<br />");
    $r->print("<strong>Files selected from other directories:</strong><br />");
    foreach (&Apache::lonnet::files_not_in_path($ENV{'user.name'}, $ENV{'form.currentpath'})) {
        $r->print($_."<br />");
    }
}
sub upload {
    my ($r)=@_;
    my $fname=$ENV{'form.uploaddoc.filename'};
    my $filesize = (length($ENV{'form.uploaddoc'})) / 1000; #express in k (1024?)
    my $disk_quota = 20000; # expressed in k
    $fname=&Apache::lonnet::clean_filename($fname);
    my $portfolio_root = &Apache::loncommon::propath($ENV{'user.domain'},
						 $ENV{'user.name'}).
						'/userfiles/portfolio';
    # Fixme --- Move the checking for existing file to LOND error return
    my @dir_list=&Apache::lonnet::dirlist($ENV{'form.currentpath'},
					  $ENV{'user.domain'},
					  $ENV{'user.name'},$portfolio_root);
    my $found_file = 0;
    foreach my $line (@dir_list) {
        my ($filename)=split(/\&/,$line,2);
        if ($filename eq $fname){
            $found_file = 1;
        }
    }
    my $current_disk_usage = &Apache::lonnet::diskusage($ENV{'user.domain'}, $ENV{'user.name'},$portfolio_root);
    if ((($current_disk_usage/1000) + $filesize) > $disk_quota){
        $r->print('<font color="red">Unable to upload <strong>'.$fname.' (size = '.$filesize.' kilobytes</strong>. Disk quota will be exceeded.'.
                  '<br />Disk quota is '.$disk_quota.' kilobytes. Your current disk usage is '.$current_disk_usage.' kilobytes.');
    } 
    elsif ($found_file){   
        $r->print('<font color="red">Unable to upload <strong>'.$fname.'</strong>, a file by that name was found in <strong>'.$ENV{'form.currentpath'}.'</strong></font>'.
                  '<br />To upload, rename or delete existing '.$fname.' in '.$ENV{'form.currentpath'});
    } else {
        my $result=&Apache::lonnet::userfileupload('uploaddoc','',
	        	 'portfolio'.$ENV{'form.currentpath'});
        if ($result !~ m|^/uploaded/|) {
            $r->print('<font color="red"> An errror occured ('.$result.
	              ') while trying to upload '.&display_file().'</font><br />');
        }
    }
    $r->print(&done());
}

sub createdir {
    my ($r)=@_;
    my $newdir=&Apache::lonnet::clean_filename($ENV{'form.newdir'});
    if ($newdir eq '') {
    	$r->print('<font color="red">'.
	    	  &mt("Error: no directory name was provided.").
		      '</font><br />');
	    $r->print(&done());
	    return;
    } 
    my $portfolio_root = &Apache::loncommon::propath($ENV{'user.domain'},
						 $ENV{'user.name'}).
						'/userfiles/portfolio';
    my @dir_list=&Apache::lonnet::dirlist($ENV{'form.currentpath'},
					  $ENV{'user.domain'},
					  $ENV{'user.name'},$portfolio_root);
    my $found_file = 0;
    foreach my $line (@dir_list) {
        my ($filename)=split(/\&/,$line,2);
        if ($filename eq $newdir){
            $found_file = 1;
        }
    }
    if ($found_file){
    	    $r->print('<font color="red"> Unable to create a directory named <strong>'.$newdir.
    	            ' </strong>a file or directory by that name already exists.</font><br />');
    } else {
        my $result=&Apache::lonnet::mkdiruserfile($ENV{'user.name'},
	         $ENV{'user.domain'},'portfolio'.$ENV{'form.currentpath'}.$newdir);
        if ($result ne 'ok') {
    	    $r->print('<font color="red"> An errror occured ('.$result.
	    	      ') while trying to create a new directory '.&display_file().'</font><br />');
        }
    }
    if ($newdir ne $ENV{'form.newdir'}) {
        $r->print("The new directory name was changed from:<br /><strong>".$ENV{'form.newdir'}."</strong> to <strong>$newdir </strong>");  
    }
    $r->print(&done());
}

sub handler {
    # this handles file management
    my $r = shift;
    my $portfolio_root = &Apache::loncommon::propath($ENV{'user.domain'},
							 $ENV{'user.name'}).
							'/userfiles/portfolio';
    &Apache::loncommon::no_cache($r);
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    # Give the LON-CAPA page header
    $r->print('<html><head><title>'.
              &mt('Portfolio Manager').
              "</title></head>\n".
              &Apache::loncommon::bodytag('Portfolio Manager'));
    $r->rflush();
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
                                            ['selectfile','currentpath',
					     'currentfile','action','fieldname','mode','rename']);
	if (($ENV{'form.storeupl'} eq 'Upload') & (!$ENV{'form.uploaddoc.filename'})){
   	    $r->print('<font color="red"> No file was selected to upload.'.
   	            'To upload a file, click <strong>Browse...</strong>'.
   	            ', select a file, then click <strong>Upload</strong>,</font>');
	}
    if ($ENV{'form.uploaddoc.filename'}) {
	&upload($r);
    } elsif ($ENV{'form.action'} eq 'delete' && $ENV{'form.confirmed'}) {
	&delete_confirmed($r);
    } elsif ($ENV{'form.action'} eq 'delete') {
	&delete($r);
    } elsif ($ENV{'form.action'} eq 'deletedir' && $ENV{'form.confirmed'}) {
	&delete_dir_confirmed($r);
    } elsif ($ENV{'form.action'} eq 'deletedir'){
	&delete_dir($r);
    } elsif ($ENV{'form.action'} eq 'rename' && $ENV{'form.confirmed'}) {
	&rename_confirmed($r);
    } elsif ($ENV{'form.rename'}) {
        $ENV{'form.selectfile'} = $ENV{'form.rename'};
        $ENV{'form.action'} = 'rename';
	&rename($r);
    } elsif ($ENV{'form.createdir'}) {
	&createdir($r);
    } else {
	my $current_path='/';
	if ($ENV{'form.currentpath'}) {
	    $current_path = $ENV{'form.currentpath'};
	}
	my @dir_list=&Apache::lonnet::dirlist($current_path,
					    $ENV{'user.domain'},
					    $ENV{'user.name'},$portfolio_root);
	if ($dir_list[0] eq 'no_such_dir'){
	    # two main reasons for this:
            #    1) never been here, so directory structure not created
	    #    2) back-button navigation after deleting a directory
	    if ($current_path eq '/'){
	        &Apache::lonnet::mkdiruserfile($ENV{'user.name'},
					       $ENV{'user.domain'},'portfolio');
	    } else {
                # some directory that snuck in get rid of the directory
                # from the recent pulldown, just in case
		&Apache::lonhtmlcommon::remove_recent('portfolio',
						      [$current_path]);
		$current_path = '/'; # force it back to the root        
	    }
	    # now grab the directory list again, for the first time
	    @dir_list=&Apache::lonnet::dirlist($current_path,
					    $ENV{'user.domain'},
					    $ENV{'user.name'},$portfolio_root);
        }
	# need to know if directory is empty so it can be removed if desired
	my $is_empty=(@dir_list == 2);
	&display_common($r,$current_path,$is_empty,\@dir_list);
	if ($ENV{"form.mode"} eq 'selectfile'){
	    &select_files($r);
            &display_file_select($r,$current_path,$is_empty,\@dir_list);
	} else {
	    &display_directory($r,$current_path,$is_empty,\@dir_list);
	}
	$r->print("</body>\n</html>\n");
	return OK;
    }
}
1;
__END__
